@@ -86,8 +86,8 @@ static struct io_u *fio_guasi_event(struct thread_data *td, int event)
 	return io_u;
 }
 
-static int fio_guasi_getevents(struct thread_data *td, int min, int max,
-			       struct timespec *t)
+static int fio_guasi_getevents(struct thread_data *td, unsigned int min,
+			       unsigned int max, struct timespec *t)
 {
 	struct guasi_data *ld = td->io_ops->data;
 	int n, r;
@@ -141,6 +141,10 @@ static void fio_guasi_queued(struct thread_data *td, struct io_u **io_us, int nr
 	struct io_u *io_u;
 	struct timeval now;
 
+	if (!fio_fill_issue_time(td))
+		return;
+
+	io_u_mark_submit(td, nr);
 	fio_gettime(&now, NULL);
 	for (i = 0; i < nr; i++) {
 		io_u = io_us[i];
@@ -212,7 +216,6 @@ static void fio_guasi_cleanup(struct thread_data *td)
 		free(ld->reqs);
 		free(ld->io_us);
 		free(ld);
-		td->io_ops->data = NULL;
 	}
 	GDBG_PRINT(("fio_guasi_cleanup(%p) DONE\n", ld));
 }
