@@ -201,7 +201,7 @@ static int fio_splice_write(struct thread_data *td, struct io_u *io_u)
 static int fio_spliceio_queue(struct thread_data *td, struct io_u *io_u)
 {
 	struct spliceio_data *sd = td->io_ops->data;
-	int ret;
+	int uninitialized_var(ret);
 
 	fio_ro_check(td, io_u);
 
@@ -232,8 +232,12 @@ static int fio_spliceio_queue(struct thread_data *td, struct io_u *io_u)
 			io_u->error = errno;
 	}
 
-	if (io_u->error)
+	if (io_u->error) {
 		td_verror(td, io_u->error, "xfer");
+		if (io_u->error == EINVAL)
+			log_err("fio: looks like splice doesn't work on this"
+					" file system\n");
+	}
 
 	return FIO_Q_COMPLETED;
 }
@@ -246,7 +250,6 @@ static void fio_spliceio_cleanup(struct thread_data *td)
 		close(sd->pipe[0]);
 		close(sd->pipe[1]);
 		free(sd);
-		td->io_ops->data = NULL;
 	}
 }
 
