@@ -55,6 +55,7 @@ static struct fio_mutex *startup_mutex;
 static volatile int fio_abort;
 static int exit_value;
 static struct itimerval itimer;
+static pthread_t gtod_thread;
 
 struct io_log *agg_io_log[2];
 
@@ -170,13 +171,15 @@ static void set_sig_handlers(void)
 
 static inline int should_check_rate(struct thread_data *td)
 {
-	/*
-	 * No minimum rate set, always ok
-	 */
-	if (!td->o.ratemin && !td->o.rate_iops_min)
-		return 0;
+	struct thread_options *o = &td->o;
 
-	return 1;
+	/*
+	 * If some rate setting was given, we need to check it
+	 */
+	if (o->rate || o->ratemin || o->rate_iops || o->rate_iops_min)
+		return 1;
+
+	return 0;
 }
 
 /*
@@ -672,7 +675,7 @@ sync_done:
 				int left;
 
 				if (td->o.thinktime_spin)
-					__usec_sleep(td->o.thinktime_spin);
+					usec_spin(td->o.thinktime_spin);
 
 				left = td->o.thinktime - td->o.thinktime_spin;
 				if (left)
@@ -940,8 +943,11 @@ static void *thread_main(void *data)
 	td->io_hist_tree = RB_ROOT;
 
 	td_set_runstate(td, TD_INITIALIZED);
+	dprint(FD_MUTEX, "up startup_mutex\n");
 	fio_mutex_up(startup_mutex);
+	dprint(FD_MUTEX, "wait on td->mutex\n");
 	fio_mutex_down(td->mutex);
+	dprint(FD_MUTEX, "done waiting on td->mutex\n");
 
 	/*
 	 * the ->mutex mutex is now no longer used, close it to avoid
@@ -964,6 +970,18 @@ static void *thread_main(void *data)
 		goto err;
 	}
 
+	/*
+	 * If we have a gettimeofday() thread, make sure we exclude that
+	 * thread from this job
+	 */
+	if (td->o.gtod_cpu) {
+		fio_cpu_clear(&td->o.cpumask, td->o.gtod_cpu);
+		if (fio_setaffinity(td) == -1) {
+			td_verror(td, errno, "cpu_set_affinity");
+			goto err;
+		}
+	}
+
 	if (td->ioprio_set) {
 		if (ioprio_set(IOPRIO_WHO_PROCESS, 0, td->ioprio) == -1) {
 			td_verror(td, errno, "ioprio_set");
@@ -1066,12 +1084,27 @@ static void *thread_main(void *data)
 	td->ts.io_bytes[0] = td->io_bytes[0];
 	td->ts.io_bytes[1] = td->io_bytes[1];
 
-	if (td->ts.bw_log)
-		finish_log(td, td->ts.bw_log, "bw");
-	if (td->ts.slat_log)
-		finish_log(td, td->ts.slat_log, "slat");
-	if (td->ts.clat_log)
-		finish_log(td, td->ts.clat_log, "clat");
+	if (td->ts.bw_log) {
+		if (td->o.bw_log_file) {
+			finish_log_named(td, td->ts.bw_log,
+						td->o.bw_log_file, "bw");
+		} else
+			finish_log(td, td->ts.bw_log, "bw");
+	}
+	if (td->ts.slat_log) {
+		if (td->o.lat_log_file) {
+			finish_log_named(td, td->ts.slat_log,
+						td->o.lat_log_file, "clat");
+		} else
+			finish_log(td, td->ts.slat_log, "slat");
+	}
+	if (td->ts.clat_log) {
+		if (td->o.lat_log_file) {
+			finish_log_named(td, td->ts.clat_log,
+						td->o.lat_log_file, "clat");
+		} else
+			finish_log(td, td->ts.clat_log, "clat");
+	}
 	if (td->o.exec_postrun) {
 		if (system(td->o.exec_postrun) < 0)
 			log_err("fio: postrun %s failed\n", td->o.exec_postrun);
@@ -1088,6 +1121,12 @@ err:
 	close_ioengine(td);
 	cleanup_io_u(td);
 
+	if (td->o.cpumask_set) {
+		int ret = fio_cpuset_exit(&td->o.cpumask);
+
+		td_verror(td, ret, "fio_cpuset_exit");
+	}
+
 	/*
 	 * do this very late, it will log file closing as well
 	 */
@@ -1217,6 +1256,46 @@ reaped:
 		terminate_threads(TERMINATE_ALL);
 }
 
+static void *gtod_thread_main(void *data)
+{
+	fio_mutex_up(startup_mutex);
+
+	/*
+	 * As long as we have jobs around, update the clock. It would be nice
+	 * to have some way of NOT hammering that CPU with gettimeofday(),
+	 * but I'm not sure what to use outside of a simple CPU nop to relax
+	 * it - we don't want to lose precision.
+	 */
+	while (threads) {
+		fio_gtod_update();
+		nop;
+	}
+
+	return NULL;
+}
+
+static int fio_start_gtod_thread(void)
+{
+	int ret;
+
+	ret = pthread_create(&gtod_thread, NULL, gtod_thread_main, NULL);
+	if (ret) {
+		log_err("Can't create gtod thread: %s\n", strerror(ret));
+		return 1;
+	}
+
+	ret = pthread_detach(gtod_thread);
+	if (ret) {
+		log_err("Can't detatch gtod thread: %s\n", strerror(ret));
+		return 1;
+	}
+
+	dprint(FD_MUTEX, "wait on startup_mutex\n");
+	fio_mutex_down(startup_mutex);
+	dprint(FD_MUTEX, "done waiting on startup_mutex\n");
+	return 0;
+}
+
 /*
  * Main function for kicking off and reaping jobs, as needed.
  */
@@ -1229,6 +1308,9 @@ static void run_threads(void)
 	if (fio_pin_memory())
 		return;
 
+	if (fio_gtod_offload && fio_start_gtod_thread())
+		return;
+
 	if (!terse_output) {
 		printf("Starting ");
 		if (nr_thread)
@@ -1332,15 +1414,21 @@ static void run_threads(void)
 			nr_started++;
 
 			if (td->o.use_thread) {
+				int ret;
+
 				dprint(FD_PROCESS, "will pthread_create\n");
-				if (pthread_create(&td->thread, NULL,
-						   thread_main, td)) {
-					perror("pthread_create");
+				ret = pthread_create(&td->thread, NULL,
+							thread_main, td);
+				if (ret) {
+					log_err("pthread_create: %s\n",
+							strerror(ret));
 					nr_started--;
 					break;
 				}
-				if (pthread_detach(td->thread) < 0)
-					perror("pthread_detach");
+				ret = pthread_detach(td->thread);
+				if (ret)
+					log_err("pthread_detach: %s",
+							strerror(ret));
 			} else {
 				pid_t pid;
 				dprint(FD_PROCESS, "will fork\n");
@@ -1352,7 +1440,9 @@ static void run_threads(void)
 				} else if (i == fio_debug_jobno)
 					*fio_debug_jobp = pid;
 			}
+			dprint(FD_MUTEX, "wait on startup_mutex\n");
 			fio_mutex_down(startup_mutex);
+			dprint(FD_MUTEX, "done waiting on startup_mutex\n");
 		}
 
 		/*
