@@ -19,7 +19,7 @@
 #include "smalloc.h"
 #include "filehash.h"
 
-static char fio_version_string[] = "fio 1.23";
+static char fio_version_string[] = "fio 1.24";
 
 #define FIO_RANDSEED		(0xb1899bedUL)
 
@@ -27,7 +27,7 @@ static char **ini_file;
 static int max_jobs = MAX_JOBS;
 static int dump_cmdline;
 
-struct thread_data def_thread;
+static struct thread_data def_thread;
 struct thread_data *threads = NULL;
 
 int exitall_on_terminate = 0;
@@ -214,6 +214,23 @@ static int fixup_options(struct thread_data *td)
 {
 	struct thread_options *o = &td->o;
 
+#ifndef FIO_HAVE_PSHARED_MUTEX
+	if (!td->o.use_thread) {
+		log_info("fio: this platform does not support process shared"
+			 " mutexes, forcing use of threads. Use the 'thread'"
+			 " option to get rid of this warning.\n");
+		td->o.use_thread = 1;
+	}
+#endif
+
+#ifndef FIO_HAVE_CPU_AFFINITY
+	if (td->o.gtod_cpu) {
+		log_err("fio: platform must support CPU affinity for"
+			"gettimeofday() offloading\n");
+		return 1;
+	}
+#endif
+
 	if (read_only && td_write(td)) {
 		log_err("fio: job <%s> has write bit set, but fio is in"
 			" read-only mode\n", td->o.name);
@@ -878,6 +895,7 @@ struct debug_level debug_levels[] = {
 	{ .name = "parse",	.shift = FD_PARSE },
 	{ .name = "diskutil",	.shift = FD_DISKUTIL },
 	{ .name = "job",	.shift = FD_JOB },
+	{ .name = "mutex",	.shift = FD_MUTEX },
 	{ .name = NULL, },
 };
 
@@ -1104,5 +1122,11 @@ int parse_options(int argc, char *argv[])
 		return 1;
 	}
 
+	if (def_thread.o.gtod_offload) {
+		fio_gtod_init();
+		fio_gtod_offload = 1;
+		fio_gtod_cpu = def_thread.o.gtod_cpu;
+	}
+
 	return 0;
 }
