@@ -4,12 +4,19 @@
  */
 #include <stdio.h>
 #include <stdlib.h>
+#include <libgen.h>
 #include <assert.h>
-#include "list.h"
+#include "flist.h"
 #include "fio.h"
 
 static const char iolog_ver2[] = "fio version 2 iolog";
 
+void queue_io_piece(struct thread_data *td, struct io_piece *ipo)
+{
+	flist_add_tail(&ipo->list, &td->io_log_list);
+	td->total_io_size += ipo->len;
+}
+
 void log_io_u(struct thread_data *td, struct io_u *io_u)
 {
 	const char *act[] = { "read", "write", "sync" };
@@ -19,7 +26,9 @@ void log_io_u(struct thread_data *td, struct io_u *io_u)
 	if (!td->o.write_iolog_file)
 		return;
 
-	fprintf(td->iolog_f, "%s %s %llu %lu\n", io_u->file->file_name, act[io_u->ddir], io_u->offset, io_u->buflen);
+	fprintf(td->iolog_f, "%s %s %llu %lu\n", io_u->file->file_name,
+						act[io_u->ddir], io_u->offset,
+						io_u->buflen);
 }
 
 void log_file(struct thread_data *td, struct fio_file *f,
@@ -32,6 +41,13 @@ void log_file(struct thread_data *td, struct fio_file *f,
 	if (!td->o.write_iolog_file)
 		return;
 
+
+	/*
+	 * this happens on the pre-open/close done before the job starts
+	 */
+	if (!td->iolog_f)
+		return;
+
 	fprintf(td->iolog_f, "%s %s\n", f->file_name, act[what]);
 }
 
@@ -53,35 +69,65 @@ static void iolog_delay(struct thread_data *td, unsigned long delay)
 	usec_sleep(td, delay);
 }
 
+static int ipo_special(struct thread_data *td, struct io_piece *ipo)
+{
+	struct fio_file *f;
+	int ret;
+
+	/*
+	 * Not a special ipo
+	 */
+	if (ipo->ddir != DDIR_INVAL)
+		return 0;
+
+	f = td->files[ipo->fileno];
+
+	switch (ipo->file_action) {
+	case FIO_LOG_OPEN_FILE:
+		ret = td_io_open_file(td, f);
+		if (!ret) {
+			free(ipo);
+			break;
+		}
+		td_verror(td, ret, "iolog open file");
+		return -1;
+	case FIO_LOG_CLOSE_FILE:
+		td_io_close_file(td, f);
+		break;
+	case FIO_LOG_UNLINK_FILE:
+		unlink(f->file_name);
+		break;
+	default:
+		log_err("fio: bad file action %d\n", ipo->file_action);
+		break;
+	}
+
+	return 1;
+}
+
 int read_iolog_get(struct thread_data *td, struct io_u *io_u)
 {
 	struct io_piece *ipo;
 
-	while (!list_empty(&td->io_log_list)) {
-		ipo = list_entry(td->io_log_list.next, struct io_piece, list);
-		list_del(&ipo->list);
+	while (!flist_empty(&td->io_log_list)) {
+		int ret;
 
-		/*
-		 * invalid ddir, this is a file action
-		 */
-		if (ipo->ddir == DDIR_INVAL) {
-			struct fio_file *f = &td->files[ipo->fileno];
+		ipo = flist_entry(td->io_log_list.next, struct io_piece, list);
+		flist_del(&ipo->list);
 
-			if (ipo->file_action == FIO_LOG_OPEN_FILE) {
-				assert(!td_io_open_file(td, f));
-				free(ipo);
-				continue;
-			} else if (ipo->file_action == FIO_LOG_CLOSE_FILE) {
-				td_io_close_file(td, f);
-				free(ipo);
-				continue;
-			}
+		ret = ipo_special(td, ipo);
+		if (ret < 0) {
+			free(ipo);
+			break;
+		} else if (ret > 0) {
+			free(ipo);
+			continue;
 		}
 
 		io_u->offset = ipo->offset;
 		io_u->buflen = ipo->len;
 		io_u->ddir = ipo->ddir;
-		io_u->file = &td->files[ipo->fileno];
+		io_u->file = td->files[ipo->fileno];
 		get_file(io_u->file);
 
 		dprint(FD_IO, "iolog: get %llu/%lu/%s\n", io_u->offset,
@@ -108,6 +154,12 @@ void prune_io_piece_log(struct thread_data *td)
 		rb_erase(n, &td->io_hist_tree);
 		free(ipo);
 	}
+
+	while (!flist_empty(&td->io_hist_list)) {
+		ipo = flist_entry(td->io_hist_list.next, struct io_piece, list);
+		flist_del(&ipo->list);
+		free(ipo);
+	}
 }
 
 /*
@@ -132,10 +184,9 @@ void log_io_piece(struct thread_data *td, struct io_u *io_u)
 	 * For both these cases, just reading back data in the order we
 	 * wrote it out is the fastest.
 	 */
-	if (!td_random(td) || !td->o.overwrite ||
-	     (io_u->file->flags & FIO_FILE_NOSORT)) {
-		INIT_LIST_HEAD(&ipo->list);
-		list_add_tail(&ipo->list, &td->io_hist_list);
+	if (!td_random(td) || !td->o.overwrite) {
+		INIT_FLIST_HEAD(&ipo->list);
+		flist_add_tail(&ipo->list, &td->io_hist_list);
 		return;
 	}
 
@@ -197,7 +248,8 @@ static int read_iolog2(struct thread_data *td, FILE *f)
 		struct io_piece *ipo;
 		int r;
 
-		r = sscanf(p, "%256s %256s %llu %u", fname, act, &offset, &bytes);
+		r = sscanf(p, "%256s %256s %llu %u", fname, act, &offset,
+									&bytes);
 		if (r == 4) {
 			/*
 			 * Check action first
@@ -209,7 +261,8 @@ static int read_iolog2(struct thread_data *td, FILE *f)
 			else if (!strcmp(act, "sync"))
 				rw = DDIR_SYNC;
 			else {
-				log_err("fio: bad iolog file action: %s\n",act);
+				log_err("fio: bad iolog file action: %s\n",
+									act);
 				continue;
 			}
 		} else if (r == 2) {
@@ -226,23 +279,24 @@ static int read_iolog2(struct thread_data *td, FILE *f)
 				fileno = get_fileno(td, fname);
 				file_action = FIO_LOG_CLOSE_FILE;
 			} else {
-				log_err("fio: bad iolog file action: %s\n",act);
+				log_err("fio: bad iolog file action: %s\n",
+									act);
 				continue;
 			}
 		} else {
 			log_err("bad iolog2: %s", p);
 			continue;
 		}
-			
+
 		if (rw == DDIR_READ)
 			reads++;
 		else if (rw == DDIR_WRITE) {
-			writes++;
 			/*
 			 * Don't add a write for ro mode
 			 */
 			if (read_only)
 				continue;
+			writes++;
 		} else if (rw != DDIR_SYNC && rw != DDIR_INVAL) {
 			log_err("bad ddir: %d\n", rw);
 			continue;
@@ -253,7 +307,7 @@ static int read_iolog2(struct thread_data *td, FILE *f)
 		 */
 		ipo = malloc(sizeof(*ipo));
 		memset(ipo, 0, sizeof(*ipo));
-		INIT_LIST_HEAD(&ipo->list);
+		INIT_FLIST_HEAD(&ipo->list);
 		ipo->offset = offset;
 		ipo->len = bytes;
 		ipo->ddir = rw;
@@ -263,8 +317,7 @@ static int read_iolog2(struct thread_data *td, FILE *f)
 			ipo->fileno = fileno;
 			ipo->file_action = file_action;
 		}
-		list_add_tail(&ipo->list, &td->io_log_list);
-		td->total_io_size += bytes;
+		queue_io_piece(td, ipo);
 	}
 
 	free(str);
@@ -272,76 +325,8 @@ static int read_iolog2(struct thread_data *td, FILE *f)
 	free(fname);
 
 	if (writes && read_only) {
-		log_err("fio: <%s> skips replay of %d writes due to read-only\n", td->o.name, writes);
-		writes = 0;
-	}
-
-	if (!reads && !writes)
-		return 1;
-	else if (reads && !writes)
-		td->o.td_ddir = TD_DDIR_READ;
-	else if (!reads && writes)
-		td->o.td_ddir = TD_DDIR_WRITE;
-	else
-		td->o.td_ddir = TD_DDIR_RW;
-
-	return 0;
-}
-
-/*
- * Read version 1 iolog data.
- */
-static int read_iolog(struct thread_data *td, FILE *f)
-{
-	unsigned long long offset;
-	unsigned int bytes;
-	char *str, *p;
-	int reads, writes;
-	int rw;
-
-	/*
-	 * Read in the read iolog and store it, reuse the infrastructure
-	 * for doing verifications.
-	 */
-	str = malloc(4096);
-	reads = writes = 0;
-	while ((p = fgets(str, 4096, f)) != NULL) {
-		struct io_piece *ipo;
-
-		if (sscanf(p, "%d,%llu,%u", &rw, &offset, &bytes) != 3) {
-			log_err("bad iolog: %s\n", p);
-			continue;
-		}
-		if (rw == DDIR_READ)
-			reads++;
-		else if (rw == DDIR_WRITE) {
-			writes++;
-			/*
-			 * Don't add a write for ro mode
-			 */
-			if (read_only)
-				continue;
-		} else if (rw != DDIR_SYNC) {
-			log_err("bad ddir: %d\n", rw);
-			continue;
-		}
-
-		ipo = malloc(sizeof(*ipo));
-		memset(ipo, 0, sizeof(*ipo));
-		INIT_LIST_HEAD(&ipo->list);
-		ipo->offset = offset;
-		ipo->len = bytes;
-		ipo->ddir = (enum fio_ddir) rw;
-		if (bytes > td->o.max_bs[rw])
-			td->o.max_bs[rw] = bytes;
-		list_add_tail(&ipo->list, &td->io_log_list);
-		td->total_io_size += bytes;
-	}
-
-	free(str);
-
-	if (writes && read_only) {
-		log_err("fio: <%s> skips replay of %d writes due to read-only\n", td->o.name, writes);
+		log_err("fio: <%s> skips replay of %d writes due to"
+			" read-only\n", td->o.name, writes);
 		writes = 0;
 	}
 
@@ -386,16 +371,8 @@ static int init_iolog_read(struct thread_data *td)
 	if (!strncmp(iolog_ver2, buffer, strlen(iolog_ver2)))
 		ret = read_iolog2(td, f);
 	else {
-		/*
-		 * seek back to the beginning
-		 */
-		if (fseek(f, 0, SEEK_SET) < 0) {
-			td_verror(td, errno, "iolog read");
-			log_err("fio: unable to read iolog\n");
-			return 1;
-		}
-
-		ret = read_iolog(td, f);
+		log_err("fio: iolog version 1 is no longer supported\n");
+		ret = 1;
 	}
 
 	fclose(f);
@@ -411,7 +388,7 @@ static int init_iolog_write(struct thread_data *td)
 	FILE *f;
 	unsigned int i;
 
-	f = fopen(td->o.write_iolog_file, "w+");
+	f = fopen(td->o.write_iolog_file, "a");
 	if (!f) {
 		perror("fopen write iolog");
 		return 1;
@@ -478,24 +455,33 @@ void __finish_log(struct io_log *log, const char *name)
 	unsigned int i;
 	FILE *f;
 
-	f = fopen(name, "w");
+	f = fopen(name, "a");
 	if (!f) {
 		perror("fopen log");
 		return;
 	}
 
-	for (i = 0; i < log->nr_samples; i++)
-		fprintf(f, "%lu, %lu, %u\n", log->log[i].time, log->log[i].val, log->log[i].ddir);
+	for (i = 0; i < log->nr_samples; i++) {
+		fprintf(f, "%lu, %lu, %u\n", log->log[i].time, log->log[i].val,
+						log->log[i].ddir);
+	}
 
 	fclose(f);
 	free(log->log);
 	free(log);
 }
 
+void finish_log_named(struct thread_data *td, struct io_log *log,
+		       const char *prefix, const char *postfix)
+{
+	char file_name[256], *p;
+
+	snprintf(file_name, 200, "%s_%s.log", prefix, postfix);
+	p = basename(file_name);
+	__finish_log(log, p);
+}
+
 void finish_log(struct thread_data *td, struct io_log *log, const char *name)
 {
-	char file_name[256];
-
-	snprintf(file_name, 200, "client%d_%s.log", td->thread_number, name);
-	__finish_log(log, file_name);
+	finish_log_named(td, log, td->o.name, name);
 }
