@@ -246,12 +246,29 @@ static int str_cpumask_cb(void *data, unsigned int *val)
 {
 	struct thread_data *td = data;
 	unsigned int i;
+	long max_cpu;
+	int ret;
 
-	CPU_ZERO(&td->o.cpumask);
+	ret = fio_cpuset_init(&td->o.cpumask);
+	if (ret < 0) {
+		log_err("fio: cpuset_init failed\n");
+		td_verror(td, ret, "fio_cpuset_init");
+		return 1;
+	}
 
-	for (i = 0; i < sizeof(int) * 8; i++)
-		if ((1 << i) & *val)
-			CPU_SET(*val, &td->o.cpumask);
+	max_cpu = sysconf(_SC_NPROCESSORS_ONLN);
+
+	for (i = 0; i < sizeof(int) * 8; i++) {
+		if ((1 << i) & *val) {
+			if (i > max_cpu) {
+				log_err("fio: CPU %d too large (max=%ld)\n", i,
+								max_cpu);
+				return 1;
+			}
+			dprint(FD_PARSE, "set cpu allowed %d\n", i);
+			fio_cpu_set(&td->o.cpumask, i);
+		}
+	}
 
 	td->o.cpumask_set = 1;
 	return 0;
@@ -261,23 +278,68 @@ static int str_cpus_allowed_cb(void *data, const char *input)
 {
 	struct thread_data *td = data;
 	char *cpu, *str, *p;
+	long max_cpu;
+	int ret = 0;
 
-	CPU_ZERO(&td->o.cpumask);
+	ret = fio_cpuset_init(&td->o.cpumask);
+	if (ret < 0) {
+		log_err("fio: cpuset_init failed\n");
+		td_verror(td, ret, "fio_cpuset_init");
+		return 1;
+	}
 
 	p = str = strdup(input);
 
 	strip_blank_front(&str);
 	strip_blank_end(str);
 
+	max_cpu = sysconf(_SC_NPROCESSORS_ONLN);
+
 	while ((cpu = strsep(&str, ",")) != NULL) {
+		char *str2, *cpu2;
+		int icpu, icpu2;
+
 		if (!strlen(cpu))
 			break;
-		CPU_SET(atoi(cpu), &td->o.cpumask);
+
+		str2 = cpu;
+		icpu2 = -1;
+		while ((cpu2 = strsep(&str2, "-")) != NULL) {
+			if (!strlen(cpu2))
+				break;
+
+			icpu2 = atoi(cpu2);
+		}
+
+		icpu = atoi(cpu);
+		if (icpu2 == -1)
+			icpu2 = icpu;
+		while (icpu <= icpu2) {
+			if (icpu >= FIO_MAX_CPUS) {
+				log_err("fio: your OS only supports up to"
+					" %d CPUs\n", (int) FIO_MAX_CPUS);
+				ret = 1;
+				break;
+			}
+			if (icpu > max_cpu) {
+				log_err("fio: CPU %d too large (max=%ld)\n",
+							icpu, max_cpu);
+				ret = 1;
+				break;
+			}
+	
+			dprint(FD_PARSE, "set cpu allowed %d\n", icpu);
+			fio_cpu_set(&td->o.cpumask, icpu);
+			icpu++;
+		}
+		if (ret)
+			break;
 	}
 
 	free(p);
-	td->o.cpumask_set = 1;
-	return 0;
+	if (!ret)
+		td->o.cpumask_set = 1;
+	return ret;
 }
 #endif
 
@@ -438,6 +500,28 @@ static int str_lockfile_cb(void *data, const char *str)
 	return 0;
 }
 
+static int str_write_bw_log_cb(void *data, const char *str)
+{
+	struct thread_data *td = data;
+
+	if (str)
+		td->o.bw_log_file = strdup(str);
+
+	td->o.write_bw_log = 1;
+	return 0;
+}
+
+static int str_write_lat_log_cb(void *data, const char *str)
+{
+	struct thread_data *td = data;
+
+	if (str)
+		td->o.lat_log_file = strdup(str);
+
+	td->o.write_lat_log = 1;
+	return 0;
+}
+
 static int str_gtod_reduce_cb(void *data, int *il)
 {
 	struct thread_data *td = data;
@@ -452,6 +536,16 @@ static int str_gtod_reduce_cb(void *data, int *il)
 	return 0;
 }
 
+static int str_gtod_cpu_cb(void *data, int *il)
+{
+	struct thread_data *td = data;
+	int val = *il;
+
+	td->o.gtod_cpu = val;
+	td->o.gtod_offload = 1;
+	return 0;
+}
+
 #define __stringify_1(x)	#x
 #define __stringify(x)		__stringify_1(x)
 
@@ -636,7 +730,7 @@ static struct fio_option options[] = {
 		.alias	= "iodepth_batch_submit",
 		.type	= FIO_OPT_INT,
 		.off1	= td_var_offset(iodepth_batch),
-		.help	= "Number of IO to submit in one go",
+		.help	= "Number of IO buffers to submit in one go",
 		.parent	= "iodepth",
 		.minval	= 1,
 		.def	= "1",
@@ -645,7 +739,7 @@ static struct fio_option options[] = {
 		.name	= "iodepth_batch_complete",
 		.type	= FIO_OPT_INT,
 		.off1	= td_var_offset(iodepth_batch_complete),
-		.help	= "Number of IO to retrieve in one go",
+		.help	= "Number of IO buffers to retrieve in one go",
 		.parent	= "iodepth",
 		.minval	= 0,
 		.def	= "1",
@@ -1266,14 +1360,16 @@ static struct fio_option options[] = {
 	},
 	{
 		.name	= "write_bw_log",
-		.type	= FIO_OPT_STR_SET,
+		.type	= FIO_OPT_STR,
 		.off1	= td_var_offset(write_bw_log),
+		.cb	= str_write_bw_log_cb,
 		.help	= "Write log of bandwidth during run",
 	},
 	{
 		.name	= "write_lat_log",
-		.type	= FIO_OPT_STR_SET,
+		.type	= FIO_OPT_STR,
 		.off1	= td_var_offset(write_lat_log),
+		.cb	= str_write_lat_log_cb,
 		.help	= "Write log of latency during run",
 	},
 	{
@@ -1341,6 +1437,12 @@ static struct fio_option options[] = {
 		.parent	= "gtod_reduce",
 		.def	= "0",
 	},
+	{
+		.name	= "gtod_cpu",
+		.type	= FIO_OPT_INT,
+		.cb	= str_gtod_cpu_cb,
+		.help	= "Setup dedicated gettimeofday() thread on this CPU",
+	},
 	{
 		.name = NULL,
 	},
