@@ -236,7 +236,7 @@ static int __handle_option(struct fio_option *o, const char *ptr, void *data,
 	dprint(FD_PARSE, "__handle_option=%s, type=%d, ptr=%s\n", o->name,
 							o->type, ptr);
 
-	if (!ptr && o->type != FIO_OPT_STR_SET) {
+	if (!ptr && o->type != FIO_OPT_STR_SET && o->type != FIO_OPT_STR) {
 		fprintf(stderr, "Option %s requires an argument\n", o->name);
 		return 1;
 	}
@@ -474,6 +474,7 @@ static struct fio_option *get_option(const char *opt,
 		*ret = '\0';
 		ret = (char *) opt;
 		(*post)++;
+		strip_blank_end(ret);
 		o = find_option(options, ret);
 	} else {
 		o = find_option(options, opt);
@@ -487,22 +488,23 @@ static int opt_cmp(const void *p1, const void *p2)
 {
 	struct fio_option *o1, *o2;
 	char *s1, *s2, *foo;
-	int ret;
+	int prio1, prio2;
 
 	s1 = strdup(*((char **) p1));
 	s2 = strdup(*((char **) p2));
 
 	o1 = get_option(s1, fio_options, &foo);
 	o2 = get_option(s2, fio_options, &foo);
-
-	if ((!o1 && o2) || (o1 && !o2))
-		ret = 0;
-	else
-		ret = o2->prio - o1->prio;
+	
+	prio1 = prio2 = 0;
+	if (o1)
+		prio1 = o1->prio;
+	if (o2)
+		prio2 = o2->prio;
 
 	free(s1);
 	free(s2);
-	return ret;
+	return prio2 - prio1;
 }
 
 void sort_options(char **opts, struct fio_option *options, int num_opts)
@@ -519,7 +521,7 @@ int parse_cmd_option(const char *opt, const char *val,
 
 	o = find_option(options, opt);
 	if (!o) {
-		fprintf(stderr, "Bad option %s\n", opt);
+		fprintf(stderr, "Bad option <%s>\n", opt);
 		return 1;
 	}
 
@@ -588,7 +590,7 @@ int parse_option(const char *opt, struct fio_option *options, void *data)
 
 	o = get_option(tmp, options, &post);
 	if (!o) {
-		fprintf(stderr, "Bad option %s\n", tmp);
+		fprintf(stderr, "Bad option <%s>\n", tmp);
 		free(tmp);
 		return 1;
 	}
